#include "Vector.h"
#include "Record.h"
#include <stdlib.h>


using namespace std;

/*
 * Costruttore senza parametri, provvede a fissare un valore iniziale per tutti
 * i campi dell'oggetto a DIM_INIZIALE e ad allocare di conseguenza la giusta
 * capacità del vettore
 */
Vettore::Vettore(){
    dimIniziale = DIM_INIZIALE;
    dimAttuale = DIM_INIZIALE;
    incremento = DIM_INIZIALE;
    sizeAttuale = 0;
    p = new Record[dimIniziale];
}

/*
 * Costruttore con due parametri, per indicare la dimensione iniziale
 * e lo spazio da aggiungere in caso ri riallocazione.
 * 
 */
Vettore::Vettore(int dimIniziale, int incremento){
    if (dimIniziale <= 0){
        p = NULL;
        this->dimIniziale = 0;
        this->dimAttuale = 0;
        this->incremento = 0;
    } else {
        p = new Record[dimIniziale];
        this->dimIniziale = dimIniziale;
        this->dimAttuale = dimIniziale;
        this->incremento = incremento;
    }
    this->sizeAttuale = 0;
}
/*
 * Costruttore di copia: genera un nuovo vettore, identico ma distinto rispetto
 * al vettore usato come esempio.
 */
Vettore::Vettore(const Vettore& v){
    dimIniziale = v.dimIniziale;
    dimAttuale = v.dimAttuale;
    sizeAttuale = v.sizeAttuale;
    incremento = v.incremento;
    p = new Record[v.dimAttuale];
    for(int i = 0; i < v.sizeAttuale; i++){
        p[i] = v.p[i];
    }
}

/*
 * Prima dealloca lo spazio utilizzato e poi rialloca inizializzando tutti i 
 * campi del vettore
 */
void Vettore::clear(){
    delete []p;
    dimAttuale = dimIniziale;
    p = new Record[dimIniziale];
    sizeAttuale = 0;
}


/* Prima versione dell'operatore di assegnazione
 * Quale è migliore!?
 *
Vettore Vettore::operator=(Vettore v){
    delete[] p;
    dimIniziale = v.dimIniziale;
    dimAttuale = v.dimAttuale;
    sizeAttuale = v.sizeAttuale;
    incremento = v.incremento;
    p = new Complesso[v.dimAttuale];
    for(int i = 0; i < ){
        p[i] = v.p[i];
    }
    return *this;
}
 */

/* Seconda versione dell'operatore di assegnazione
 * Quale è migliore!?
 */
Vettore Vettore::operator=(Vettore v){
    // Ripulisci il vettore corrente (che si trova a sinistra dell'operatore di
    // assegnazione
    clear();
    // ricopia nel vettore corrente tutti gli elementi di v
    for(int i = 0; i < v.size(); i++){
        // o in alternativa si poteva scrivere: this->add(v.p[i]);
        add(v.p[i]);
    }
    // Restituisce una copia del vettore per assegnazioni in cascata
    return *this;
}

/*
 * Somma tra vettori: è intesa come accodamento 
 * Prima versione operatore di somma.
 *
Vettore Vettore::operator+(Vettore v){
    Vettore app(0,0); // Perchè conviene 0,0!?
    app = *this;
    for(int i = 0; i < v.size(); i++){
        app.add(v.p[i]);
    }
    return app;
}
 */

/* Seconda versione dell'operatore di somma
 * Quale è migliore!?
 */

Vettore Vettore::operator+(Vettore v){
    Vettore app(0,0); // Perchè conviene 0,0!?
    app.dimAttuale = this->dimAttuale + v.dimAttuale;
    app.sizeAttuale = this->sizeAttuale + v.sizeAttuale;
    app.incremento = this->incremento;
    app.dimIniziale = this->dimIniziale;
    app.p = new Record[app.dimAttuale];
    for(int i = 0; i < this->sizeAttuale; i++){
        app.p[i] = this->p[i];
    }
    for(int i = 0; i < v.sizeAttuale; i++){
        app.p[i + this->sizeAttuale] = v.p[i];
    }
    return app;
}

Vettore::~Vettore(){
    delete[] p;
}

Record Vettore::get(int index){
    return p[index];
}

void Vettore::add(Record a){
    // Se la dimensiona attuale ha raggiunto la capacità massima, rialloca
    if (sizeAttuale >= dimAttuale){
        dimAttuale = dimAttuale + incremento;
        Record *temp = new Record[dimAttuale];
        for(int i = 0; i < sizeAttuale; i++){
            temp[i] = p[i];
        }
        delete[] p;
        p = temp;
    }
    // In ogni caso, inserisci in coda
    p[sizeAttuale++] = a;

}

int Vettore::size(){
    return sizeAttuale;
}

/*
 * La stampa procede utilizzando la capacità di produrre una stampa, che compete
 * ai singoli oggetti, individuabili con p[i]
 */
void Vettore::stampa(){
    for(int i = 0; i < sizeAttuale; i++){
        p[i].stampa();
    }
}
